<?php
/**
 * The LDAP class attempts to change a user's password stored in an
 * LDAP directory service.
 *
 * $Horde: passwd/lib/Driver/ldap.php,v 1.30 2004/08/19 10:40:20 jan Exp $
 *
 * @author  Mike Cochrane <mike@graftonhall.co.nz>
 * @author  Tjeerd van der Zee <admin@xar.nl>
 * @author  Mattias Webjrn Eriksson <mattias@webjorn.org>
 * @author  Eric Jon Rostetter <eric.rostetter@physics.utexas.edu>
 * @version $Revision$
 * @package Passwd
 */
class Passwd_Driver_ldap extends Passwd_Driver {

    /**
     * LDAP connection handle.
     * @var integer $_ds
     */
    var $_ds;

    /**
     * Constructs a new Passwd_Driver_ldap object.
     *
     * @param array $params  A hash containing connection parameters.
     */
    function Passwd_Driver_ldap($params = array())
    {
        $this->_params['host']            = isset($params['host']) ? $params['host'] : 'localhost';
        $this->_params['sslhost']         = isset($params['sslhost']) ? $params['sslhost'] : '';
        $this->_params['port']            = isset($params['port']) ? $params['port'] : 389;
        $this->_params['encryption']      = isset($params['encryption']) ? $params['encryption'] : 'crypt';
        $this->_params['show_encryption'] = isset($params['show_encryption']) ? $params['show_encryption'] : 'true';
        $this->_params['uid']             = isset($params['uid']) ? $params['uid'] : 'uid';
        $this->_params['basedn']          = isset($params['basedn']) ? $params['basedn'] : '';
        $this->_params['realm']           = isset($params['realm']) ? $params['realm'] : '';
        $this->_params['tls']             = isset($params['tls']) ? $params['tls'] : null;
        $this->_params['attribute']       = isset($params['attribute']) ? $params['attribute'] : 'userPassword';
        $this->_params['shadowlastchange']= isset($params['shadowlastchange']) ? $params['shadowlastchange'] : 'shadowLastChange';
        $this->_params['shadowmin']       = isset($params['shadowmin']) ? $params['shadowmin'] : 'shadowMin';
    }

    /**
     * Does an ldap connect and binds as the guest user or as the optional
     * userdn.
     *
     * @param string $userdn       The dn to use when binding non-anonymously.
     * @param string $oldpassword  The password for $userdn.
     * @param boolean $ssl         The SSL trigger.
     *
     * @return boolean  True or False based on success of connect and bind.
     */
    function _connect($userdn = null, $password = null, $ssl = false)
    {
        if ($ssl && empty($this->_params['tls'])) {
            $this->_ds = ldap_connect('ldaps://' . $this->_params['sslhost']);
        } else {
            $this->_ds = ldap_connect($this->_params['host'], $this->_params['port']);
        }
        if (!$this->_ds) {
            return PEAR::raiseError(_("Could not connect to ldap server"));
        }

        if (ldap_set_option($this->_ds, LDAP_OPT_PROTOCOL_VERSION, 3) &&
            $this->_params['tls']) {
            if (!ldap_start_tls($this->_ds)) {
                return PEAR::raiseError(_("ldap_start_tls failed"));
            }
        }

        // Try to bind as the current userdn with password.
        $result = false;
        if (!is_null($userdn)) {
            $result = @ldap_bind($this->_ds, $userdn, $password);
        }

        // If unable to bind as current user, still try to bind as
        // guest.
        if (!$result) {
            $result = @ldap_bind($this->_ds);
        }

        // If none of the bind attempts succeed, return error.
        if (!$result) {
            return PEAR::raiseError(_("Could not bind to ldap server"));
        }

        return true;
    }

    /**
     * Looks up and returns the user's dn.
     *
     * @param string $user    The username of the user.
     * @param string $passw   The password of the user.
     * @param string $realm   The realm (domain) name of the user.
     * @param string $basedn  The ldap basedn.
     * @param string $uid     The ldap uid.
     *
     * @return string  The ldap dn for the user.
     */
    function _lookupdn($user, $passw)
    {
        // Construct username@realm to connect as if 'realm' parameter
        // is set.
        $urealm = '';
        if (!empty($this->_params['realm'])) {
            $urealm = $user . '@' . $this->_params['realm'];
        }

        // Bind as current user. _connect will try as guest if no user
        // realm is found or auth error.
        $this->_connect($urealm, $passw);

        // Construct search.
        $search = $this->_params['uid'] . '=' . $user;

        // Get userdn.
        $result = ldap_search($this->_ds, $this->_params['basedn'], $search);
        $entry = ldap_first_entry($this->_ds, $result);
        if ($entry === false) {
            return PEAR::raiseError(_("User not found."));
        }

        $userdn = ldap_get_dn($this->_ds, $entry);

        return $userdn;
    }
    
    /**
     * Checks for shadowLastChange and shadowMin support and returns their
     * values.
     *
     * @param string $user   The username of the user.
     * @param string $passw  The password of the user.
     *
     * @return array  Hash with keys being "shadowlastchange" and "shadowmin"
     *                and containing their respective values or false for no
     *                support.
     */
    function _lookupShadow($user, $passw)
    {
        // Init the return array
        $lookupshadow = array('shadowlastchange' => false, 
                              'shadowmin' => false);
    
        // Construct username@realm to connect as if 'realm' parameter is set.
        $urealm = $user;
        if (!empty($this->_params['realm'])) {
            $urealm .= '@' . $this->_params['realm'];
        }

        // Bind as current user. _connect will try as guest if no user
        // realm is found or auth error.
        $this->_connect($urealm, $passw);

        // Construct search.
        $search = $this->_params['uid'] . '=' . $user;

        // Get userdn.
        $result = ldap_search($this->_ds, $this->_params['basedn'], $search);
        $entry = ldap_first_entry($this->_ds, $result);
        if ($entry !== false) {
            $information = ldap_get_values($this->_ds, $entry, $this->_params['shadowlastchange']);
            if ($information) {
                $lookupshadow['shadowlastchange'] = $information[0];
            }
        
            $information = ldap_get_values($this->_ds, $entry, $this->_params['shadowmin']);
            if ($information) {
                $lookupshadow['shadowmin'] = $information[0];
            }
        }
        
        return $lookupshadow;
    }

    /**
     * Changes the user's password.
     *
     * @param string $username      The user for which to change the password.
     * @param string $old_password  The old (current) user password.
     * @param string $new_password  The new user password to set.
     *
     * @return boolean  True or false based on success of the change.
     */
    function changePassword($username, $old_password, $new_password)
    {
        // Get the user's dn.
        global $conf;

        if ($conf['hooks']['userdn']) {
            $userdn = Horde::callHook('_passwd_hook_userdn', array(Auth::getAuth()));
        } else {
            $userdn = $this->_lookupdn($username, $old_password);
            if (is_a($userdn, 'PEAR_Error')) {
                return $userdn;
            }
        }

        $lookupshadow = $this->_lookupShadow($username, $old_password);
        
        // Check if we may change the password
        if ($lookupshadow['shadowlastchange'] && $lookupshadow['shadowmin']) {
            if ($lookupshadow['shadowlastchange'] + $lookupshadow['shadowmin'] > (time() / 86400)) {
                return PEAR::raiseError(_("Minimum password age has not yet expired"));
            }
        }

        // Construct username@realm to connect if 'realm' parameter is
        // set.  Looks like the _passwd_hook_username hook, but here
        // we use a configurable parameter as realm.  This is a
        // safeguard that if unable to lookup the user's DN we still
        // have a chance to authenticate.
        if (empty($userdn) && !empty($this->_params['realm'])) {
            $userdn = $username . '@' . $this->_params['realm'];
        }

        // Connect as the user.
        $res = $this->_connect($userdn, $old_password, !empty($this->_params['sslhost']));
        if (is_a($res, 'PEAR_Error')) {
            if ($res->getMessage() == _("Could not bind to ldap server")) {
                return PEAR::raiseError(_("Incorrect Password"));
            }
            return $res;
        }
        
        // Change the user's password and update lastchange
        $new_details[$this->_params['attribute']] = $this->encryptPassword($new_password);
        
        if ($lookupshadow['shadowlastchange']) {
            $new_details[$this->_params['shadowlastchange']] = floor(time() / 86400);
        }
        
        if (!ldap_mod_replace($this->_ds, $userdn, $new_details)) {
            return PEAR::raiseError(ldap_error($this->_ds));
        }

        return true;
    }

}
