<?php

require_once dirname(__FILE__) . '/spamd.php';

/**
 * Sam storage implementation for PHP's PEAR database abstraction layer.
 *
 * Required parameters:
 * ====================
 *   'database'  --  The name of the database.
 *   'hostspec'  --  The hostname of the database server.
 *   'password'  --  The password associated with 'username'.
 *   'phptype'   --  The database type (ie. 'pgsql', 'mysql, etc.).
 *   'protocol'  --  The communication protocol ('tcp', 'unix', etc.).
 *   'username'  --  The username with which to connect to the database.
 *
 * Optional preferences:
 * =====================
 *   'table'     --  The name of the Sam options table in 'database'.
 *                   DEFAULT: 'userpref'
 *
 * Required by some database implementations:
 * ==========================================
 *   'options'  --  Additional options to pass to the database.
 *   'port'     --  The port on which to connect to the database.
 *   'tty'      --  The TTY on which to connect to the database.

 * The table structure can be created by the scripts/drivers/spamd_*.sql
 * script appropriate for your database, or modified from one that is
 * available.
 *
 * $Horde: sam/lib/Driver/spamd_sql.php,v 1.41 2005/01/03 14:35:26 jan Exp $
 *
 * Copyright 2003-2005 Chris Bowlby <excalibur@hub.org>
 * Copyright 2003-2005 Max Kalika <max@horde.org>
 * Copyright 2004-2005 Jan Schneider <jan@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Chris Bowlby <excalibur@hub.org>
 * @author  Max Kalika <max@horde.org>
 * @author  Jan Schneider <jan@horde.org>
 * @version $Revision$
 * @since   SAM 0.0.1
 * @package Sam
 */
class SAM_Driver_spamd_sql extends SAM_Driver_spamd {

    /**
     * Handle for the current database connection.
     *
     * @var object DB $_db
     */
    var $_db;

    /**
     * Boolean indicating whether or not we're connected to the SQL server.
     *
     * @var boolean $_connected
     */
    var $_connected = false;

    /**
     * Constructs a new SQL storage object.
     *
     * @access public
     *
     * @param string $user               The user who owns these SPAM options.
     * @param optional array $params     A hash containing connection
     *                                   parameters.
     */
    function SAM_Driver_spamd_sql($user, $params = array())
    {
        global $conf;

        $this->_user = $user;
        $this->_capabilities[] = 'global_defaults';
        $this->_params = array_merge($conf['sql'], $params);
    }

    /**
     * Retrieve an option set from the storage backend.
     *
     * @access private
     *
     * @param optional boolean $defaults Whether to retreive the global
     *                                   defaults instead of user options.
     *
     * @return mixed    Array of option-value pairs or a PEAR_Error object
     *                  on failure.
     */
    function _retrieve($defaults = false)
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();

        if ($defaults) {
            $user = isset($this->_params['global_user'])
                    ? $this->_params['global_user'] : '@GLOBAL';
        } else {
            $user = $this->_user;
        }

        /* Build the SQL query. */
        $query = sprintf('SELECT * FROM %s WHERE username = %s',
                         $this->_params['table'],
                         $this->_db->quote($user));

        $return = array();

        /* Log the query at a DEBUG log level. */
        Horde::logMessage(sprintf('SAM_Driver_spamd_sql::_retrieve(): %s', $query),
                          __FILE__, __LINE__, PEAR_LOG_DEBUG);

        /* Execute the query. */
        $result = $this->_db->query($query);
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        /* Loop through rows, retrieving options. */
        while (($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) && !is_a($row, 'PEAR_Error')) {
            $attribute = $this->_mapOptionToAttribute($row['preference']);

            if (isset($return[$attribute])) {
                if (!is_array($return[$attribute])) {
                    $return[$attribute] = array($return[$attribute]);
                }
                if (!in_array($row['value'], $return[$attribute])) {
                    $return[$attribute][] = $row['value'];
                }
            } else {
                $return[$attribute] = $row['value'];
            }
        }
        $result->free();

        if (is_a($row, 'PEAR_Error')) {
            return $row;
        }

        return $return;
    }

    /**
     * Retreives the global defaults and user options and stores them in the
     * appropriate member array (options or defaults).
     *
     * @access public
     *
     * @return mixed    True on success or a PEAR_Error object on failure.
     */
    function retrieve()
    {
        /* Load defaults for any options the user hasn't already overridden. */
        $defaults = $this->_retrieve(true);
        if (!is_a($defaults, 'PEAR_Error')) {
            $this->_defaults = $defaults;
        } else {
            return $defaults;
        }

        $this->_options = $this->_defaults;
        $options = $this->_retrieve();
        if (!is_a($options, 'PEAR_Error')) {
            $this->_options = array_merge($this->_options, $options);
        } else {
            return $options;
        }

        return true;
    }

    /**
     * Store an option set from the appropriate member array (options or
     * defaults) to the storage backend.
     *
     * @access private
     *
     * @param optional boolean $defaults Whether to store the global defaults
     *                                   instead of user options.
     *
     * @return mixed    True on success or a PEAR_Error object on failure.
     */
    function _store($defaults = false)
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();

        if ($defaults) {
            $store = $this->_defaults;
            $user = isset($this->_params['global_user'])
                        ? $this->_params['global_user'] : '@GLOBAL';
        } else {
            $store = $this->_options;
            $user = $this->_user;
        }

        foreach ($store as $attribute => $value) {
            $option = $this->_mapAttributeToOption($attribute);

            /* Delete the option if it is the same as the default */
            if (!$defaults && isset($this->_defaults[$attribute]) &&
                $this->_defaults[$attribute] === $value) {
                $query = sprintf('DELETE FROM %s WHERE username = %s ' .
                                 'AND preference = %s;',
                                 $this->_params['table'],
                                 $this->_db->quote($user),
                                 $this->_db->quote($option));
                /* Log the query at a DEBUG log level. */
                Horde::logMessage(sprintf('SAM_Driver_spamd_sql::_store(): %s', $query),
                                  __FILE__, __LINE__, PEAR_LOG_DEBUG);
                $this->_db->query($query);
                continue;
            }

            if (is_array($value)) {
                $query = sprintf('DELETE FROM %s WHERE username = %s ' .
                                 'AND preference = %s;',
                                 $this->_params['table'],
                                 $this->_db->quote($user),
                                 $this->_db->quote($option));
                /* Log the query at a DEBUG log level. */
                Horde::logMessage(sprintf('SAM_Driver_spamd_sql::_store(): %s', $query),
                                  __FILE__, __LINE__, PEAR_LOG_DEBUG);
                $this->_db->query($query);

                foreach ($value as $address) {
                    /* Don't save email addresses already in defaults. */
                    if (!$defaults && isset($this->_defaults[$attribute]) &&
                        ((is_array($this->_defaults[$attribute]) &&
                          in_array($address, $this->_defaults[$attribute])) ||
                         $this->_defaults[$attribute] === $address)) {
                        continue;
                    }

                    $query = sprintf('INSERT INTO %s (username, preference, value) ' .
                                     'VALUES (%s, %s, %s);',
                                     $this->_params['table'],
                                     $this->_db->quote($user),
                                     $this->_db->quote($option),
                                     $this->_db->quote($address));
                    /* Log the query at a DEBUG log level. */
                    Horde::logMessage(sprintf('SAM_Driver_spamd_sql::_store(): %s', $query),
                                      __FILE__, __LINE__, PEAR_LOG_DEBUG);
                    $result = $this->_db->query($query);
                    if (is_a($result, 'PEAR_Error')) {
                        return $result;
                    }
                }
            } else {
                $query = sprintf('SELECT 1 FROM %s WHERE username = %s ' .
                                 'AND preference = %s',
                                 $this->_params['table'],
                                 $this->_db->quote($user),
                                 $this->_db->quote($option));

                $result = $this->_db->getOne($query);
                if (is_a($result, 'PEAR_Error')) {
                    return $result;
                }

                if (is_null($result)) {
                    $query = sprintf('INSERT INTO %s (username, preference, value) ' .
                                     'VALUES (%s, %s, %s)',
                                     $this->_params['table'],
                                     $this->_db->quote($user),
                                     $this->_db->quote($option),
                                     $this->_db->quote($value));
                } else {
                    $query = sprintf('UPDATE %s SET VALUE = %s WHERE username = %s ' .
                                     'AND preference = %s',
                                     $this->_params['table'],
                                     $this->_db->quote($value),
                                     $this->_db->quote($user),
                                     $this->_db->quote($option));
                }

                /* Log the query at a DEBUG log level. */
                Horde::logMessage(sprintf('SAM_Driver_spamd_sql::_store(): %s', $query),
                                  __FILE__, __LINE__, PEAR_LOG_DEBUG);
                $result = $this->_db->query($query);
                if (is_a($result, 'PEAR_Error')) {
                    return $result;
                }
            }
        }

        return true;
    }

    /**
     * Stores the global defaults or user options from the appropriate
     * member array (options or defaults).
     *
     * @access public
     *
     * @param optional boolean $defaults Whether to store the global defaults
     *                                   instead of user options.
     *
     * @return mixed    True on success or a PEAR_Error object on failure.
     */
    function store($defaults = false)
    {
        return $this->_store($defaults);
    }

    /**
     * Attempts to open a persistent connection to the SQL server.
     *
     * @access private
     *
     * @return mixed    True on success or a PEAR_Error object on failure.
     */
    function _connect()
    {
        if (!$this->_connected) {
            Horde::assertDriverConfig($this->_params, 'spamd_sql',
                array('phptype', 'hostspec', 'username', 'database'),
                'SAM backend', 'backends.php', '$backends');
            if (!isset($this->_params['table'])) {
                $this->_params['table'] = 'userpref';
            }

            /* Connect to the SQL server using the supplied parameters. */
            require_once 'DB.php';
            $this->_db = &DB::connect($this->_params,
                                      array('persistent' => !empty($this->_params['persistent'])));
            if (is_a($this->_db, 'PEAR_Error')) {
                Horde::fatal($this->_db, __FILE__, __LINE__);
            }

            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);

            $this->_connected = true;
        }

        return true;
    }

    /**
     * Disconnect from the SQL server and clean up the connection.
     *
     * @return boolean  True on success, false on failure.
     */
    function _disconnect()
    {
        if ($this->_connected) {
            $this->_connected = false;
            return $this->_db->disconnect();
        }

        return true;
    }

}
