<?php
/**
 * $Horde: vacation/lib/Driver/ldap.php,v 1.6 2005/01/03 14:35:41 jan Exp $
 *
 * Copyright 2001-2005 Eric Rostetter <eric.rostetter@physics.utexas.edu>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/bsdl.php.
 *
 * Vacation_Driver_ldap:: implements the Vacation_Driver API for LDAP-compliant
 * mail servers (such as Exim).
 *
 * Parameters:
 * (required)
 *   host       - hostname of the LDAP server
 *   port       - port number of the LDAP service
 *   basedn     - base DN of the user directory
 *   uid        - attribute to use for uid
 *   vacation   - attribute to use for storing the vacation message
 *   active     - attribute which determines if the vacation message is active
 * (optional)
 *   userdn     - another way of specifying the user DN (instead of
 *                constructing it from uid+basedn).
 *   version    - Protocol version for the LDAP server (PHP defaults
 *                to version 2. OpenLDAP >= 2.1.4 uses version 3, and
 *                so must be set explicitly).
 *
 * @author  Eric Rostetter <eric.rostetter@physics.utexas.edu>
 * @version $Revision$
 * @package Vacation
 */
class Vacation_Driver_ldap extends Vacation_Driver {

    /** Pointer to the ldap connection. */
    var $_ds;

    /** Hash containing connection parameters. */
    var $_params;

    /** The error string returned to the user if an error occurs. */
    var $err_str;

    /**
     * Constructs a new ftp dot-forward Vacation_Driber object.
     *
     * @param array  $params    A hash containing connection parameters.
     */
    function Vacation_Driver_ldap($params = array())
    {
        $this->_params = $params;
    }

    /**
     * Do an ldap connect and bind as the guest user or as the optional userdn.
     *
     * @param    $userdn      The optional dn to use when binding non-anonymously.
     * @param    $oldpassword The optional password for $userdn.
     *
     * @return   boolean   True or False based on success of connect and bind.
     *
     */
    function _connect($userdn = null, $password = null, $realm = 'default')
    {
        $this->_ds = ldap_connect($this->_params[$realm]['host'], $this->_params[$realm]['port']);
        if (!$this->_ds) {
            return PEAR::raiseError(_("Could not connect to ldap server"));
        }
        if (array_key_exists('version', $this->_params[$realm])) {
             ldap_set_option($this->_ds, LDAP_OPT_PROTOCOL_VERSION,
					$this->_params[$realm]['version']);
        }

        if (!is_null($userdn)) {
            $result = @ldap_bind($this->_ds, $userdn, $password);
        } else {
            $result = @ldap_bind($this->_ds);
        }

        if (!$result) {
            return PEAR::raiseError(_("Could not bind to ldap server"));
        }

        return true;
    }

    /**
     * Close the ldap connection.
     */
    function _disconnect()
    {
        @ldap_close($this->_ds);
    }

    /**
     * Check if the realm has a specific configuration.  If not, try to fall
     * back on the default configuration.  If still not a valid configuration
     * then exit with an error.
     *
     * @param string    $realm      The realm of the user, or "default" if none.
     *                              Note: passed by reference so we can change
     *                              it's value!
     *
     */
    function checkConfig(&$realm)
    {
        // If no realm passed in, or no host config for the realm passed in,
        // then we fall back to the default realm
        if (empty($realm) || empty($this->_params[$realm]['server'])) {
            $realm = 'default';
        }

        // If still no host/port, then we have a misconfigured module.
        if (empty($this->_params[$realm]['host']) ||
            empty($this->_params[$realm]['port']) ) {
            $this->err_str = _("The module is not properly configured!");
            return false;
        }
        return true;
    }

    /**
     * Lookup and return the user's dn.
     *
     * @param  $user     The username of the user.
     * @param  $realm    The realm (domain) name of the user.
     * @param  $basedn   The ldap basedn.
     * @param  $uid      The ldap uid.
     *
     * @return string    The ldap dn for the user.
     */
    function _lookupdn($user, $realm)
    {
        // bind as guest
        $this->_connect();

        // construct search
        $search = $this->_params[$realm]['uid'] . '=' . $user;
        if (!empty($this->_params[$realm]['realm'])) {
            $search .= '@' . $this->_params[$realm]['realm'];
        }

        // get userdn
        $result = ldap_search($this->_ds, $this->_params[$realm]['basedn'], $search);
        $entry = ldap_first_entry($this->_ds, $result);
        if ($entry === false) {
            $this->_disconnect();
            return PEAR::raiseError(_("User not found."));
        }
        $userdn = ldap_get_dn($this->_ds, $entry);

        // disconnect from ldap server
        $this->_disconnect();

        return $userdn;
    }

    /**
     * Set the vacation notice up.
     * @param string    $user       The username to enable vacation for.
     * @param string    $realm      The realm of the user.
     * @param string    $target     The message to install.
     * @return boolean  Returns true on success, false on error.
     */
    function setVacation($user, $realm, $pass, $message)
    {
        // Make sure the configuration file is correct
        if (!$this->checkConfig($realm)) {
            return false;
        }

        // get the user's dn
        if (array_key_exists('userdn', $this->_params[$realm])) {
            $userdn = $this->_params[$realm]['userdn'];
        } else {
            $userdn = $this->_lookupdn($user, $realm);
            if (is_a($userdn, 'PEAR_Error')) {
                return $userdn;
            }
        }

        // connect as the user
        $res = $this->_connect($userdn, $pass, $realm);
        if (is_a($res, 'PEAR_Error')) {
            $this->err_str = $res->getMessage();
            $this->err_str .= ' - ' .  _("Check your password");
            return false;
        }

        // Prepare the message. \n->\n\n and UTF-8 encode.
        $message = str_replace("\r\n", "\\n", $message);
        $message = mb_convert_encoding($message, 'UTF-8', 'ISO-8859-1');

        // change the user's vacation.
        $newDetails[$this->_params[$realm]['vacation']] = $message;
        $newDetails[$this->_params[$realm]['active']] = 'Y';
        $res = ldap_mod_replace($this->_ds, $userdn, $newDetails);
        $attribs = array($this->_params[$realm]['vacation'], $this->_params[$realm]['active']);
        $value = $this->_getVacation($userdn, $attribs);
        if (!$res) {
            $res = PEAR::raiseError(ldap_error($this->_ds));
        }

        // disconnect from ldap server
        $this->_disconnect();

        return true;
    }

    function _getUserDetails($user, $realm = 'default', $pass)
    {
        // Make sure the configuration file is correct.
        if (!$this->checkConfig($realm)) {
            return false;
        }

        // Get the user's dn.
        if (array_key_exists('userdn', $this->_params[$realm])) {
            $userdn = $this->_params[$realm]['userdn'];
        } else {
            $userdn = $this->_lookupdn($user, $realm);
            if (is_a($userdn, 'PEAR_Error')) {
                return $userdn;
            }
        }

        // Connect as the user.
        $res = $this->_connect($userdn, $pass, $realm);
        if (is_a($res, 'PEAR_Error')) {
            $this->_disconnect();
            if ($res->getMessage() == _("Could not bind to ldap server")) {
                return PEAR::raiseError(_("Incorrect Password"));
            }
            return $res;
        }

        $attribs = array($this->_params[$realm]['vacation'], $this->_params[$realm]['active']);
        $vac = $this->_getVacation($userdn, $attribs);

        // Prepare the message. \n->\n\n and UTF-8 encode.
        $vac['message'] = str_replace("\\\\n", "\r\n", $vac['message']);
        $vac['message'] = mb_convert_encoding($vac['message'], 'ISO-8859-1', 'UTF-8');

        return $vac;
    }

    function _getVacation($userdn, $attribs)
    {
        $sr = ldap_search($this->_ds, $userdn, 'uid=*', $attribs);
        $entry = ldap_first_entry($this->_ds, $sr);
        if (!$entry) {
            return false;
        }
        $value = ldap_get_values($this->_ds, $entry, $attribs[0]);
        if ($value['count'] == 0) {
            return false;
        }
        $retval['message'] = $value[0];
        $value = ldap_get_values($this->_ds, $entry, $attribs[1]);
        if ($value['count'] == 0) {
            return false;
        }
        $retval['vacation'] = $value[0];
        return $retval;
    }

    /**
     * Deactivate the vacation notice.
     * NB: This does not delete the vacation message, just marks it as
     * disabled.
     *
     * @param string    $user       The user to disable vacation notices for.
     * @param string    $realm      The realm of the user.
     * @param string    $pass       The password of the user.
     *
     * @return boolean  Returns true on success, false on error.
     */
    function unsetVacation($user, $realm, $pass)
    {
        // Make sure the configuration file is correct.
        if (!$this->checkConfig($realm)) {
            return false;
        }

        // Get the user's dn.
        if (isset($this->_params[$realm]['userdn'])) {
            $userdn = $this->_params[$realm]['userdn'];
        } else {
            $userdn = $this->_lookupdn($user, $realm);
            if (is_a($userdn, 'PEAR_Error')) {
                return $userdn;
            }
        }

        // Connect as the user.
        $res = $this->_connect($userdn, $pass, $realm);
        if (is_a($res, 'PEAR_Error')) {
            $this->_disconnect();
            if ($res->getMessage() == _("Could not bind to ldap server")) {
                return PEAR::raiseError(_("Incorrect Password"));
            }
            return $res;
        }

        // Set the vacation message to inactive.
        $newDetails[$this->_params[$realm]['active']] = 'N';
        $res = ldap_mod_replace($this->_ds, $userdn, $newDetails);
        if (!$res) {
            $res = PEAR::raiseError(ldap_error($this->_ds));
        }

        // Disconnect from LDAP server.
        $this->_disconnect();

        return true;
    }

}
